@@ -33,6 +33,9 @@
 #	include <config.h>
 #endif
 
+#include <synfig/localization.h>
+#include <synfig/general.h>
+
 #include <ETL/stringf>
 #include "trgt_ffmpeg.h"
 #include <cstdio>
