@@ -30,19 +30,28 @@
 #	include <config.h>
 #endif
 
-#include "layer.h"
-#include "canvas.h"
 #include <cassert>
-#include "exception.h"
-#include "time.h"
-#include "context.h"
-#include <synfig/layers/layer_pastecanvas.h>
-#include <synfig/valuenodes/valuenode_const.h>
-#include <synfig/valuenodes/valuenode_scale.h>
-#include "loadcanvas.h"
-#include "filesystemnative.h"
+#include <cstring>
+
 #include <sigc++/bind.h>
 
+#include "time.h"
+
+#include "general.h"
+#include <synfig/localization.h>
+
+#include "canvas.h"
+#include "context.h"
+#include "exception.h"
+#include "filesystemnative.h"
+#include "importer.h"
+#include "layer.h"
+#include "loadcanvas.h"
+
+#include "layers/layer_pastecanvas.h"
+#include "valuenodes/valuenode_const.h"
+#include "valuenodes/valuenode_scale.h"
+
 #endif
 
 using namespace synfig;
@@ -80,7 +89,7 @@ Canvas::Canvas(const String &id):
 	is_inline_	(false),
 	is_dirty_	(true),
 	op_flag_	(false),
-	grow_value	(0.0)
+	outline_grow(0.0)
 {
 	identifier_.file_system = FileSystemNative::instance();
 	_CanvasCounter::counter++;
@@ -187,13 +196,13 @@ Canvas::empty()const
 Layer::Handle &
 Canvas::back()
 {
-	return *(CanvasBase::end()-1);
+	return *(end()-1);
 }
 
 const Layer::Handle &
 Canvas::back()const
 {
-	return *(CanvasBase::end()-1);
+	return *(end()-1);
 }
 
 IndependentContext
@@ -214,6 +223,27 @@ Canvas::get_context(const Context &parent_context)const
 	return get_context(parent_context.get_params());
 }
 
+void
+Canvas::get_context_sorted(const ContextParams &params, CanvasBase &out_queue, Context &out_context) const
+{
+	multimap<Real, Layer::Handle> layers;
+	int index = 0;
+	for(const_iterator i = begin(); i != end(); ++i, ++index)
+	{
+		assert(*i);
+		// TODO: the 1.0001 constant should be somehow user defined
+		Real depth = (*i)->get_z_depth()*1.0001 + (Real)index;
+		layers.insert(pair<Real, Layer::Handle>(depth, *i));
+	}
+
+	out_queue.clear();
+	for(multimap<Real, Layer::Handle>::const_iterator i = layers.begin(); i != layers.end(); ++i)
+		out_queue.push_back(i->second);
+	out_queue.push_back(Layer::Handle());
+
+	out_context = Context(out_queue.begin(), params);
+}
+
 const ValueNodeList &
 Canvas::value_node_list()const
 {
@@ -297,20 +327,19 @@ Canvas::set_description(const String &x)
 }
 
 void
-Canvas::set_grow_value(Real x)
+Canvas::set_outline_grow(Real x)
 {
-	if(grow_value!=x)
+	if (fabs(outline_grow - x) > 1e-8)
 	{
-		grow_value=x;
-		get_independent_context().set_dirty_outlines();
+		outline_grow = x;
+		get_independent_context().set_outline_grow(outline_grow);
 	}
-
 }
 
 Real
-Canvas::get_grow_value()const
+Canvas::get_outline_grow()const
 {
-	return grow_value;
+	return outline_grow;
 }
 
 void
@@ -850,16 +879,12 @@ Canvas::clone(const GUID& deriv_guid, bool for_export)const
 	if(is_inline() && !for_export)
 	{
 		canvas->is_inline_=true;
-		// \todo this was setting parent_=0 - is there a reason for that?
+		// \todo this was setting parent=0 - is there a reason for that?
 		// this was causing bug 1838132, where cloning an inline canvas that contains an imported image fails
 		// it was failing to ascertain the absolute pathname of the imported image, since it needs the pathname
 		// of the canvas to get that, which is stored in the parent canvas
-//		printf("%s:%d clone: setting parent_ of %lx to %lx\n", __FILE__, __LINE__, uintptr_t(canvas.get()), uintptr_t(parent().get()));
-		canvas->parent_=parent();
-//		show_canvas_ancestry(__FILE__, __LINE__, "clone(): old");
-//		canvas->show_canvas_ancestry(__FILE__, __LINE__, "clone(): new");
 		canvas->rend_desc() = rend_desc();
-		//canvas->set_inline(parent());
+		canvas->set_parent(parent());
 	}
 
 	canvas->set_guid(get_guid()^deriv_guid);
@@ -908,20 +933,14 @@ Canvas::set_inline(LooseHandle parent)
 
 	id_=_("in line");
 	is_inline_=true;
-//	printf("%s:%d set_inline: setting parent_ of %lx to %lx\n", __FILE__, __LINE__, uintptr_t(this), uintptr_t(parent.get()));
-	parent_=parent;
-//	show_canvas_ancestry(__FILE__, __LINE__, "set_inline()");
 
 	// Have the parent inherit all of the group stuff
-
 	std::map<String,std::set<etl::handle<Layer> > >::const_iterator iter;
-
 	for(iter=group_db_.begin();iter!=group_db_.end();++iter)
-	{
 		parent->group_db_[iter->first].insert(iter->second.begin(),iter->second.end());
-	}
-
 	rend_desc()=parent->rend_desc();
+
+	set_parent(parent);
 }
 
 Canvas::Handle
@@ -941,17 +960,13 @@ Canvas::new_child_canvas()
 {
 	if(is_inline() && parent_)
 		return parent_->new_child_canvas();
-//		runtime_error("You cannot create a child Canvas in an inline Canvas");
 
 	// Create a new canvas
 	children().push_back(create());
 	Canvas::Handle canvas(children().back());
 
-//	printf("%s:%d new_child_canvas: setting parent_ of %lx to %lx\n", __FILE__, __LINE__, uintptr_t(canvas.get()), uintptr_t(this));
-	canvas->parent_=this;
-//	canvas->show_canvas_ancestry(__FILE__, __LINE__, "new_child_canvas");
-
 	canvas->rend_desc()=rend_desc();
+	canvas->set_parent(this);
 
 	return canvas;
 }
@@ -961,17 +976,14 @@ Canvas::new_child_canvas(const String &id)
 {
 	if(is_inline() && parent_)
 		return parent_->new_child_canvas(id);
-//		runtime_error("You cannot create a child Canvas in an inline Canvas");
 
 	// Create a new canvas
 	children().push_back(create());
 	Canvas::Handle canvas(children().back());
 
 	canvas->set_id(id);
-//	printf("%s:%d new_child_canvas: setting parent_ of %lx to %lx\n", __FILE__, __LINE__, uintptr_t(canvas.get()), uintptr_t(this));
-	canvas->parent_=this;
-//	canvas->show_canvas_ancestry(__FILE__, __LINE__, "new_child_canvas");
 	canvas->rend_desc()=rend_desc();
+	canvas->set_parent(this);
 
 	return canvas;
 }
@@ -1000,9 +1012,7 @@ Canvas::add_child_canvas(Canvas::Handle child_canvas, const synfig::String& id)
 			child_canvas->is_inline_=false;
 		child_canvas->id_=id;
 		children().push_back(child_canvas);
-//		printf("%s:%d add_child_canvas: setting parent_ of %lx to %lx\n", __FILE__, __LINE__, uintptr_t(child_canvas.get()), uintptr_t(this));
-		child_canvas->parent_=this;
-//		child_canvas->show_canvas_ancestry(__FILE__, __LINE__, "add_child_canvas");
+		child_canvas->set_parent(this);
 	}
 
 	return child_canvas;
@@ -1021,10 +1031,25 @@ Canvas::remove_child_canvas(Canvas::Handle child_canvas)
 		throw Exception::IDNotFound(child_canvas->get_id());
 
 	children().remove(child_canvas);
+	child_canvas->set_parent(0);
+}
 
-//	printf("%s:%d remove_child_canvas: setting parent_ of %lx to 0\n", __FILE__, __LINE__, uintptr_t(child_canvas.get()));
-	child_canvas->parent_=0;
-//	child_canvas->show_canvas_ancestry(__FILE__, __LINE__, "remove_child_canvas");
+void
+Canvas::set_parent(const Canvas::LooseHandle &parent)
+{
+	parent_ = parent;
+	on_parent_set();
+}
+
+void
+Canvas::on_parent_set()
+{
+	// parent canvas is important field,
+	// so assume that canvas replaced for layers
+	for(std::list<Handle>::iterator i = children().begin(); i != children().end(); ++i)
+		(*i)->on_parent_set();
+	for(iterator i = begin(); *i; ++i)
+		(*i)->on_canvas_set();
 }
 
 void
@@ -1081,6 +1106,12 @@ Canvas::get_file_path()const
 	return dirname(file_name_);
 }
 
+FileSystem::Handle
+Canvas::get_file_system()const
+{
+	return get_identifier().file_system;
+}
+
 void
 Canvas::set_identifier(const FileSystem::Identifier &identifier)
 {
@@ -1093,7 +1124,6 @@ Canvas::get_identifier()const
 	return parent() ? parent()->get_identifier() : identifier_;
 }
 
-
 String
 Canvas::get_meta_data(const String& key)const
 {
@@ -1244,11 +1274,6 @@ synfig::optimize_layers(Time time, Context context, Canvas::Handle op_canvas, bo
 			Canvas::Handle paste_sub_canvas = paste_canvas->get_sub_canvas();
 			if(paste_sub_canvas)
 			{
-				Real parent_grow(paste_canvas->get_parent_canvas_grow_value());
-				if(paste_sub_canvas->is_inline())
-					paste_sub_canvas->set_grow_value(parent_grow+paste_canvas->get_param("outline_grow").get(Real()));
-				else
-					paste_sub_canvas->set_grow_value(0.0);
 				ContextParams params=context.get_params();
 				paste_canvas->apply_z_range_to_params(params);
 				optimize_layers(time, paste_sub_canvas->get_context(params),sub_canvas,motion_blurred);
@@ -1301,7 +1326,6 @@ synfig::optimize_layers(Time time, Context context, Canvas::Handle op_canvas, bo
 			etl::handle<Layer_PasteCanvas> new_layer =
 				etl::handle<Layer_PasteCanvas>::cast_dynamic( Layer::create(paste_canvas->get_name()) );
 			new_layer->set_optimized(true);
-			new_layer->set_muck_with_time(false);
 			if (motion_blurred)
 			{
 				Layer::DynamicParamList dynamic_param_list(paste_canvas->dynamic_param_list());
@@ -1312,7 +1336,6 @@ synfig::optimize_layers(Time time, Context context, Canvas::Handle op_canvas, bo
 			//param_list.erase("canvas");
 			new_layer->set_param_list(param_list);
 			new_layer->set_sub_canvas(sub_canvas);
-			new_layer->set_muck_with_time(true);
 			layer=new_layer;
 		}
 		else					// not a PasteCanvas - does it use blend method 'Straight'?
@@ -1367,7 +1390,7 @@ synfig::optimize_layers(Time time, Context context, Canvas::Handle op_canvas, bo
 			// (because paste will be always new layer)
 			// Oops... not always...
 			//if(dynamic_cast<Layer_PasteCanvas*>(layer.get()) != NULL)
-				composite = composite->simple_clone();
+				composite = etl::handle<Layer_Composite>::cast_dynamic(composite->simple_clone());
 			// Let's scale the amount parameter by the z depth visibility
 			ValueNode::Handle amount;
 			// First look if amount is dynamic:
@@ -1398,10 +1421,6 @@ synfig::optimize_layers(Time time, Context context, Canvas::Handle op_canvas, bo
 	for(iter2=sort_list.begin();iter2!=sort_list.end();++iter2)
 		op_canvas->push_back_simple(iter2->second);
 	op_canvas->op_flag_=true;
-	if(!context->empty() && (*context)->get_canvas())
-	{
-		op_canvas->set_grow_value((*context)->get_canvas()->get_grow_value());
-	}
 }
 
 void
