@@ -31,6 +31,9 @@
 #	include <config.h>
 #endif
 
+#include <synfig/localization.h>
+#include <synfig/general.h>
+
 #include <iostream>
 #include "version.h"
 #include "general.h"
@@ -41,7 +44,7 @@
 #include <stdexcept>
 
 // Includes used by get_binary_path():
-#ifdef WIN32
+#ifdef _WIN32
 #include <windows.h>
 #elif defined(__APPLE__)
 #include <mach-o/dyld.h>
@@ -63,6 +66,7 @@
 #include "layer.h"
 #include "valuenode.h"
 #include "soundprocessor.h"
+#include "rendering/renderer.h"
 
 #include "main.h"
 #include "loadcanvas.h"
@@ -71,6 +75,8 @@
 
 #include "mutex.h"
 
+#include <giomm.h>
+
 #ifdef HAVE_SIGNAL_H
 #include <signal.h>
 #endif
@@ -92,6 +98,20 @@ using namespace synfig;
 /* === S T A T I C S ======================================================= */
 
 static etl::reference_counter synfig_ref_count_(0);
+Main *Main::instance = NULL;
+
+class GeneralIOMutexHolder {
+private:
+	Mutex mutex;
+	bool initialized;
+public:
+	GeneralIOMutexHolder(): initialized(true) { }
+	~GeneralIOMutexHolder() { initialized = false; }
+	void lock() { if (initialized) mutex.lock(); }
+	void unlock() { if (initialized) mutex.unlock(); }
+};
+
+GeneralIOMutexHolder general_io_mutex;
 
 /* === P R O C E D U R E S ================================================= */
 
@@ -153,11 +173,11 @@ static void broken_pipe_signal (int /*sig*/)  {
 
 bool retrieve_modules_to_load(String filename,std::list<String> &modules_to_load)
 {
-	std::ifstream file(filename.c_str());
+	std::ifstream file(Glib::locale_from_utf8(filename).c_str());
 
 	if(!file)
 	{
-		// warning("Cannot open "+filename);
+		synfig::warning("Cannot open "+filename);
 		return false;
 	}
 
@@ -181,21 +201,28 @@ synfig::Main::Main(const synfig::String& basepath,ProgressCallback *cb):
 	synfig_ref_count_.reset();
 	ref_count_=synfig_ref_count_;
 
+	assert(!instance);
+	instance = this;
+
+	// Paths
+
+	root_path       = etl::dirname(basepath);
+	bin_path        = root_path  + ETL_DIRECTORY_SEPARATOR + "bin";
+	share_path      = root_path  + ETL_DIRECTORY_SEPARATOR + "share";
+	locale_path     = share_path + ETL_DIRECTORY_SEPARATOR + "locale";
+	lib_path        = root_path  + ETL_DIRECTORY_SEPARATOR + "lib";
+	lib_synfig_path = lib_path   + ETL_DIRECTORY_SEPARATOR + "synfig";
+
 	// Add initialization after this point
 
 #ifdef ENABLE_NLS
 	String locale_dir;
-	locale_dir = etl::dirname(basepath)+ETL_DIRECTORY_SEPARATOR+"share"+ETL_DIRECTORY_SEPARATOR+"locale";
-#ifdef WIN32
-	locale_dir = Glib::locale_from_utf8(locale_dir);
-#endif
+	locale_dir = locale_path;
 
-	bindtextdomain("synfig", locale_dir.c_str() );
+	bindtextdomain("synfig", Glib::locale_from_utf8(locale_path).c_str() );
 	bind_textdomain_codeset("synfig", "UTF-8");
 #endif
 
-	String prefix=etl::dirname(basepath);
-
 	unsigned int i;
 #ifdef _DEBUG
 #ifndef __APPLE__
@@ -208,6 +235,9 @@ synfig::Main::Main(const synfig::String& basepath,ProgressCallback *cb):
 #endif
 
 	//_config_search_path=new vector"string.h"();
+	
+	// Init Gio to allow use Gio::File::create_for_uri() in ValueNode_AnimatedFile::load file() function
+	Gio::init();
 
 	// Init the subsystems
 	if(cb)cb->amount_complete(0, 100);
@@ -218,12 +248,25 @@ synfig::Main::Main(const synfig::String& basepath,ProgressCallback *cb):
 
 	if(cb)cb->task(_("Starting Subsystem \"Types\""));
 	if(!Type::subsys_init())
+	{
+		SoundProcessor::subsys_stop();
 		throw std::runtime_error(_("Unable to initialize subsystem \"Types\""));
+	}
 
-	if(cb)cb->task(_("Starting Subsystem \"Modules\""));
-	if(!Module::subsys_init(prefix))
+	if(cb)cb->task(_("Starting Subsystem \"Rendering\""));
+	if(!rendering::Renderer::subsys_init())
 	{
 		Type::subsys_stop();
+		SoundProcessor::subsys_stop();
+		throw std::runtime_error(_("Unable to initialize subsystem \"Rendering\""));
+	}
+
+	if(cb)cb->task(_("Starting Subsystem \"Modules\""));
+	if(!Module::subsys_init(root_path))
+	{
+		rendering::Renderer::subsys_stop();
+		Type::subsys_stop();
+		SoundProcessor::subsys_stop();
 		throw std::runtime_error(_("Unable to initialize subsystem \"Modules\""));
 	}
 
@@ -231,7 +274,9 @@ synfig::Main::Main(const synfig::String& basepath,ProgressCallback *cb):
 	if(!Layer::subsys_init())
 	{
 		Module::subsys_stop();
+		rendering::Renderer::subsys_stop();
 		Type::subsys_stop();
+		SoundProcessor::subsys_stop();
 		throw std::runtime_error(_("Unable to initialize subsystem \"Layers\""));
 	}
 
@@ -240,7 +285,9 @@ synfig::Main::Main(const synfig::String& basepath,ProgressCallback *cb):
 	{
 		Layer::subsys_stop();
 		Module::subsys_stop();
+		rendering::Renderer::subsys_stop();
 		Type::subsys_stop();
+		SoundProcessor::subsys_stop();
 		throw std::runtime_error(_("Unable to initialize subsystem \"Targets\""));
 	}
 
@@ -250,7 +297,9 @@ synfig::Main::Main(const synfig::String& basepath,ProgressCallback *cb):
 		Target::subsys_stop();
 		Layer::subsys_stop();
 		Module::subsys_stop();
+		rendering::Renderer::subsys_stop();
 		Type::subsys_stop();
+		SoundProcessor::subsys_stop();
 		throw std::runtime_error(_("Unable to initialize subsystem \"Importers\""));
 	}
 
@@ -261,7 +310,9 @@ synfig::Main::Main(const synfig::String& basepath,ProgressCallback *cb):
 		Target::subsys_stop();
 		Layer::subsys_stop();
 		Module::subsys_stop();
+		rendering::Renderer::subsys_stop();
 		Type::subsys_stop();
+		SoundProcessor::subsys_stop();
 		throw std::runtime_error(_("Unable to initialize subsystem \"Cairo Importers\""));
 	}
 
@@ -273,7 +324,9 @@ synfig::Main::Main(const synfig::String& basepath,ProgressCallback *cb):
 		Target::subsys_stop();
 		Layer::subsys_stop();
 		Module::subsys_stop();
+		rendering::Renderer::subsys_stop();
 		Type::subsys_stop();
+		SoundProcessor::subsys_stop();
 		throw std::runtime_error(_("Unable to initialize subsystem \"ValueNodes\""));
 	}
 
@@ -289,41 +342,41 @@ synfig::Main::Main(const synfig::String& basepath,ProgressCallback *cb):
 		locations.push_back(getenv("SYNFIG_MODULE_LIST"));
 	else
 	{
-		locations.push_back("./"MODULE_LIST_FILENAME);
+		locations.push_back("./" MODULE_LIST_FILENAME);
 		if(getenv("HOME"))
 			locations.push_back(strprintf("%s/.local/share/synfig/%s", getenv("HOME"), MODULE_LIST_FILENAME));
 	#ifdef SYSCONFDIR
-		locations.push_back(SYSCONFDIR"/"MODULE_LIST_FILENAME);
+		locations.push_back(SYSCONFDIR"/" MODULE_LIST_FILENAME);
 	#endif
-		locations.push_back(prefix+ETL_DIRECTORY_SEPARATOR+"etc"+ETL_DIRECTORY_SEPARATOR+MODULE_LIST_FILENAME);
-		locations.push_back("/usr/local/etc/"MODULE_LIST_FILENAME);
+		locations.push_back(root_path + ETL_DIRECTORY_SEPARATOR + "etc" + ETL_DIRECTORY_SEPARATOR + MODULE_LIST_FILENAME);
+		locations.push_back("/usr/local/etc/" MODULE_LIST_FILENAME);
 	#ifdef __APPLE__
-		locations.push_back("/Library/Frameworks/synfig.framework/Resources/"MODULE_LIST_FILENAME);
-		locations.push_back("/Library/Synfig/"MODULE_LIST_FILENAME);
+		locations.push_back("/Library/Frameworks/synfig.framework/Resources/" MODULE_LIST_FILENAME);
+		locations.push_back("/Library/Synfig/" MODULE_LIST_FILENAME);
 		if(getenv("HOME"))
 			locations.push_back(strprintf("%s/Library/Synfig/%s", getenv("HOME"), MODULE_LIST_FILENAME));
 	#endif
-	#ifdef WIN32
-		locations.push_back("C:\\Program Files\\Synfig\\etc\\"MODULE_LIST_FILENAME);
-	#endif
 	}
 
 	for(i=0;i<locations.size();i++)
 		if(retrieve_modules_to_load(locations[i],modules_to_load))
 		{
-			synfig::info(_("Loading modules from %s"), locations[i].c_str());
+			synfig::info(_("Loading modules from %s"), Glib::locale_from_utf8(locations[i]).c_str());
 			if(cb)cb->task(strprintf(_("Loading modules from %s"),locations[i].c_str()));
 			break;
 		}
 
 	if (i == locations.size())
 	{
-		Importer::subsys_stop();
+		ValueNode::subsys_stop();
 		CairoImporter::subsys_stop();
+		Importer::subsys_stop();
 		Target::subsys_stop();
 		Layer::subsys_stop();
 		Module::subsys_stop();
+		rendering::Renderer::subsys_stop();
 		Type::subsys_stop();
+		SoundProcessor::subsys_stop();
 		throw std::runtime_error(strprintf(_("Unable to open module list file '%s'"), MODULE_LIST_FILENAME));
 	}
 
@@ -374,11 +427,16 @@ synfig::Main::~Main()
  	// synfig::info("Module::subsys_stop()");
 	// Module::subsys_stop();
 	// synfig::info("Exiting");
+	rendering::Renderer::subsys_stop();
 	Type::subsys_stop();
+	SoundProcessor::subsys_stop();
 
 #if defined(HAVE_SIGNAL_H) && defined(SIGPIPE)
 	signal(SIGPIPE, SIG_DFL);
 #endif
+
+	assert(instance);
+	instance = NULL;
 }
 
 static const String
@@ -405,8 +463,9 @@ synfig::error(const char *format,...)
 void
 synfig::error(const String &str)
 {
-	static Mutex mutex; Mutex::Lock lock(mutex);
+	general_io_mutex.lock();
 	cerr<<"synfig("<<getpid()<<")"<<current_time().c_str()<<_("error")<<": "<<str.c_str()<<endl;
+	general_io_mutex.unlock();
 }
 
 void
@@ -420,8 +479,9 @@ synfig::warning(const char *format,...)
 void
 synfig::warning(const String &str)
 {
-	static Mutex mutex; Mutex::Lock lock(mutex);
+	general_io_mutex.lock();
 	cerr<<"synfig("<<getpid()<<")"<<current_time().c_str()<<_("warning")<<": "<<str.c_str()<<endl;
+	general_io_mutex.unlock();
 }
 
 void
@@ -435,8 +495,9 @@ synfig::info(const char *format,...)
 void
 synfig::info(const String &str)
 {
-	static Mutex mutex; Mutex::Lock lock(mutex);
+	general_io_mutex.lock();
 	cerr<<"synfig("<<getpid()<<")"<<current_time().c_str()<<_("info")<<": "<<str.c_str()<<endl;
+	general_io_mutex.unlock();
 }
 
 // synfig::get_binary_path()
@@ -448,7 +509,7 @@ synfig::get_binary_path(const String &fallback_path)
 	
 	String result;
 
-#ifdef WIN32
+#ifdef _WIN32
 	
 	size_t buf_size = PATH_MAX - 1;
 	char* path = (char*)malloc(buf_size);
@@ -579,5 +640,8 @@ synfig::get_binary_path(const String &fallback_path)
 		// (usually should come from argv[0])
 		result = etl::absolute_path(fallback_path);
 	}
+	
+	result = Glib::locale_to_utf8(result);
+	
 	return result;
 }
