@@ -37,6 +37,7 @@
 
 #include "savecanvas.h"
 #include "general.h"
+#include <synfig/localization.h>
 #include "valuenode.h"
 #include "valuenodes/valuenode_animated.h"
 #include "valuenodes/valuenode_const.h"
@@ -57,6 +58,8 @@
 #include "paramdesc.h"
 #include "weightedvalue.h"
 #include "pair.h"
+#include "segment.h"
+#include "transformation.h"
 
 #include "zstreambuf.h"
 #include "importer.h"
@@ -613,7 +616,6 @@ xmlpp::Element* encode_dynamic_list(xmlpp::Element* root,ValueNode_DynamicList::
 
 		// process waypoints
 		{
-			typedef synfig::ValueNode_DynamicList::ListEntry::Activepoint Activepoint;
 			typedef synfig::ValueNode_DynamicList::ListEntry::ActivepointList ActivepointList;
 			String begin_sequence;
 			String end_sequence;
@@ -1086,7 +1088,7 @@ synfig::save_canvas(const FileSystem::Identifier &identifier, Canvas::ConstHandl
 
 		encode_canvas_toplevel(document.create_root_node("canvas"),canvas);
 
-		FileSystem::WriteStreamHandle stream = identifier.file_system->get_write_stream(tmp_filename);
+		FileSystem::WriteStream::Handle stream = identifier.file_system->get_write_stream(tmp_filename);
 		if (!stream)
 		{
 			synfig::error("synfig::save_canvas(): Unable to open file for write");
@@ -1094,7 +1096,7 @@ synfig::save_canvas(const FileSystem::Identifier &identifier, Canvas::ConstHandl
 		}
 
 		if (filename_extension(identifier.filename) == ".sifz")
-			stream = FileSystem::WriteStreamHandle(new ZWriteStream(stream));
+			stream = FileSystem::WriteStream::Handle(new ZWriteStream(stream));
 
 		document.write_to_stream_formatted(*stream, "UTF-8");
 
