@@ -32,6 +32,7 @@
 #endif
 
 #include <synfig/general.h>
+#include <synfig/localization.h>
 #include "valuenode_add.h"
 #include "valuenode_const.h"
 #include <stdexcept>
@@ -162,15 +163,21 @@ synfig::ValueNode_Add::operator()(Time t)const
 ValueBase
 synfig::ValueNode_Add::get_inverse(Time t, const synfig::Real &target_value) const
 {
-	return target_value / (*scalar)(t).get(Real()) - (*ref_b)(t).get(Vector::value_type());
+	return target_value / (*scalar)(t).get(Real()) - (*ref_b)(t).get(Real());
 }
 
-synfig::ValueBase
+ValueBase
 synfig::ValueNode_Add::get_inverse(Time t, const synfig::Angle &target_value) const
 {
 	return target_value / (*scalar)(t).get(Real()) - (*ref_b)(t).get(Angle());
 }
 
+ValueBase
+synfig::ValueNode_Add::get_inverse(Time t, const synfig::Vector &target_value) const
+{
+	return target_value / (*scalar)(t).get(Real()) - (*ref_b)(t).get(Vector());
+}
+
 bool
 ValueNode_Add::set_link_vfunc(int i,ValueNode::Handle value)
 {
@@ -229,12 +236,12 @@ ValueNode_Add::get_children_vocab_vfunc() const
 	LinkableValueNode::Vocab ret;
 
 	ret.push_back(ParamDesc(ValueBase(),"lhs")
-		.set_local_name(_("LHS"))
+		.set_local_name(_("Link"))
 		.set_description(_("Left Hand Side of the add"))
 	);
 
 	ret.push_back(ParamDesc(ValueBase(),"rhs")
-		.set_local_name(_("RHS"))
+		.set_local_name(_("Addition"))
 		.set_description(_("Right Hand Side of the add"))
 	);
 
