@@ -36,6 +36,7 @@
 #include "valuenode_const.h"
 #include <stdexcept>
 #include <synfig/general.h>
+#include <synfig/localization.h>
 #include "valuenode_radialcomposite.h"
 #include <synfig/vector.h>
 #include <synfig/color.h>
@@ -44,6 +45,9 @@
 #include <synfig/transformation.h>
 #include <synfig/weightedvalue.h>
 #include <synfig/pair.h>
+#include <synfig/blinepoint.h>
+#include <synfig/widthpoint.h>
+#include <synfig/dashitem.h>
 
 #endif
 
