@@ -35,6 +35,7 @@
 #include "valuenode_const.h"
 #include "valuenode_composite.h"
 #include <synfig/general.h>
+#include <synfig/localization.h>
 #include <synfig/exception.h>
 #include <vector>
 #include <list>
@@ -389,7 +390,6 @@ ValueNode_DynamicList::ListEntry::find(const Time& begin,const Time& end,std::ve
 float
 ValueNode_DynamicList::ListEntry::amount_at_time(const Time &t,bool *rising)const
 {
-	typedef synfig::ValueNode_DynamicList::ListEntry::Activepoint Activepoint;
 	typedef synfig::ValueNode_DynamicList::ListEntry::ActivepointList ActivepointList;
 
 	if(timing_info.empty())
@@ -437,7 +437,7 @@ ValueNode_DynamicList::ListEntry::new_activepoint_at_time(const Time& time)const
 bool
 ValueNode_DynamicList::ListEntry::status_at_time(const Time &t)const
 {
-	typedef synfig::ValueNode_DynamicList::ListEntry::Activepoint Activepoint;
+	//typedef synfig::ValueNode_DynamicList::ListEntry::Activepoint Activepoint;
 	typedef synfig::ValueNode_DynamicList::ListEntry::ActivepointList ActivepointList;
 
 	ActivepointList::const_iterator entry_iter;
@@ -755,6 +755,8 @@ ValueNode_DynamicList::link_name(int i)const
 int
 ValueNode_DynamicList::get_link_index_from_name(const String &name)const
 {
+	for(int i = 0; i < link_count(); ++i)
+		if (link_name(i) == name) return i;
 	throw Exception::BadLinkName(name);
 }
 
